<?php

namespace App\Domain\Cloud\Actions\DispatchToSync;

use App\Domain\Cloud\Jobs\SyncPropertiesJob;
use App\Domain\Offers\Models\Property;
use App\Domain\Support\SyncTimestampTypeEnum;
use Carbon\CarbonInterface;

class DispatchPropertiesToSyncAction implements DispatchEntityToSyncInterface
{
    public function __construct(protected DispatchEntityToSyncAction $dispatchEntityToIndexing)
    {
    }

    public function execute(): void
    {
        $this->dispatchEntityToIndexing->execute(
            type: SyncTimestampTypeEnum::CLOUD_PROPERTIES,
            queryBuilder: function (?CarbonInterface $from, CarbonInterface $to) {
                $query = Property::withTrashed()
                    ->where('cloud_fields_updated_at', '<', $to)
                    ->selectRaw('property_id as dispatch_id');

                if ($from) {
                    $query->where('cloud_fields_updated_at', '>=', $from);
                }

                return $query;
            },
            dispatch: function (array $ids) {
                SyncPropertiesJob::dispatch($ids);
            },
            logger: logger()->channel('cloud:dispatch-to-sync:properties')
        );
    }
}
