<?php

namespace App\Domain\Cloud\Actions\DispatchToSync;

use App\Domain\Cloud\Jobs\SyncPropertyDirectoryValuesJob;
use App\Domain\Offers\Models\PropertyDirectoryValue;
use App\Domain\Support\SyncTimestampTypeEnum;
use Carbon\CarbonInterface;

class DispatchPropertyDirectoryValuesToSyncAction implements DispatchEntityToSyncInterface
{
    public function __construct(protected DispatchEntityToSyncAction $dispatchEntityToIndexing)
    {
    }

    public function execute(): void
    {
        $this->dispatchEntityToIndexing->execute(
            type: SyncTimestampTypeEnum::CLOUD_PROPERTY_DIRECTORY_VALUES,
            queryBuilder: function (?CarbonInterface $from, CarbonInterface $to) {
                $query = PropertyDirectoryValue::withTrashed()
                    ->where('cloud_fields_updated_at', '<', $to)
                    ->selectRaw('directory_value_id as dispatch_id');

                if ($from) {
                    $query->where('cloud_fields_updated_at', '>=', $from);
                }

                return $query;
            },
            dispatch: function (array $ids) {
                SyncPropertyDirectoryValuesJob::dispatch($ids);
            },
            logger: logger()->channel('cloud:dispatch-to-sync:property-directory-values')
        );
    }
}
