<?php

use App\Domain\Cloud\Actions\DispatchToSync\DispatchPropertiesToSyncAction;
use App\Domain\Cloud\Jobs\SyncPropertiesJob;
use App\Domain\Offers\Models\Property;
use App\Domain\Support\Models\SyncTimestamp;
use App\Domain\Support\SyncTimestampTypeEnum;
use Illuminate\Support\Facades\Date;

use function Pest\Laravel\assertDatabaseHas;

use Tests\ComponentTestCase;

uses(ComponentTestCase::class);
uses()->group('component', 'DispatchPropertiesToSyncAction');

test("Action DispatchPropertiesToSyncAction success with SyncTimestamp", function () {
    /** @var ComponentTestCase $this */
    Date::setTestNow(now());
    /** @var SyncTimestamp $timestamp */
    $timestamp = SyncTimestamp::factory()->forType(SyncTimestampTypeEnum::CLOUD_PROPERTIES)->create([
        'last_schedule' => now()->subDay(),
    ]);

    // Properties added to job
    $timeOk = ['cloud_fields_updated_at' => $timestamp->last_schedule->addSecond()];
    $properties = Property::factory()->count(2)->create($timeOk);
    $properties->push(
        Property::factory()->trashed()->create($timeOk)
    );

    // Properties not added to job
    $timeNon = ['cloud_fields_updated_at' => $timestamp->last_schedule->subSecond()];
    Property::factory()->count(2)->create($timeNon);
    Property::factory()->trashed()->create($timeNon);

    Queue::fake();

    resolve(DispatchPropertiesToSyncAction::class)->execute();

    Queue::assertPushed(function (SyncPropertiesJob $job) use ($properties) {
        return arrayEquals($job->ids, $properties->pluck('property_id')->all());
    });

    assertDatabaseHas(SyncTimestamp::class, [
        'id' => $timestamp->id,
        'last_schedule' => now()->format((new SyncTimestamp())->getDateFormat()),
    ]);
});

test("Action DispatchPropertiesToSyncAction success without SyncTimestamp", function () {
    /** @var ComponentTestCase $this */
    Date::setTestNow(now());

    // Properties added to job
    $properties = Property::factory()->count(2)->create();

    Queue::fake();

    resolve(DispatchPropertiesToSyncAction::class)->execute();

    Queue::assertPushed(function (SyncPropertiesJob $job) use ($properties) {
        return arrayEquals($job->ids, $properties->pluck('property_id')->all());
    });

    assertDatabaseHas(SyncTimestamp::class, [
        'type' => SyncTimestampTypeEnum::CLOUD_PROPERTIES,
        'last_schedule' => now()->format((new SyncTimestamp())->getDateFormat()),
    ]);
});
