<?php

use App\Domain\Cloud\Actions\DispatchToSync\DispatchPropertyDirectoryValuesToSyncAction;
use App\Domain\Cloud\Jobs\SyncPropertyDirectoryValuesJob;
use App\Domain\Offers\Models\PropertyDirectoryValue;
use App\Domain\Support\Models\SyncTimestamp;
use App\Domain\Support\SyncTimestampTypeEnum;
use Illuminate\Support\Facades\Date;

use function Pest\Laravel\assertDatabaseHas;

use Tests\ComponentTestCase;

uses(ComponentTestCase::class);
uses()->group('component', 'DispatchPropertyDirectoryValuesToSyncAction');

test("Action DispatchPropertyDirectoryValuesToSyncAction success with SyncTimestamp", function () {
    /** @var ComponentTestCase $this */
    Date::setTestNow(now());
    /** @var SyncTimestamp $timestamp */
    $timestamp = SyncTimestamp::factory()->forType(SyncTimestampTypeEnum::CLOUD_PROPERTY_DIRECTORY_VALUES)->create([
        'last_schedule' => now()->subDay(),
    ]);

    // PropertyDirectoryValues added to job
    $timeOk = ['cloud_fields_updated_at' => $timestamp->last_schedule->addSecond()];
    $values = PropertyDirectoryValue::factory()->count(2)->create($timeOk);
    $values->push(
        PropertyDirectoryValue::factory()->trashed()->create($timeOk)
    );

    // PropertyDirectoryValues not added to job
    $timeNon = ['cloud_fields_updated_at' => $timestamp->last_schedule->subSecond()];
    PropertyDirectoryValue::factory()->count(2)->create($timeNon);
    PropertyDirectoryValue::factory()->trashed()->create($timeNon);

    Queue::fake();

    resolve(DispatchPropertyDirectoryValuesToSyncAction::class)->execute();

    Queue::assertPushed(function (SyncPropertyDirectoryValuesJob $job) use ($values) {
        return arrayEquals($job->ids, $values->pluck('directory_value_id')->all());
    });

    assertDatabaseHas(SyncTimestamp::class, [
        'id' => $timestamp->id,
        'last_schedule' => now()->format((new SyncTimestamp())->getDateFormat()),
    ]);
});

test("Action DispatchPropertyDirectoryValuesToSyncAction success without SyncTimestamp", function () {
    /** @var ComponentTestCase $this */
    Date::setTestNow(now());

    // PropertyDirectoryValues added to job
    $values = PropertyDirectoryValue::factory()->count(2)->create();

    Queue::fake();

    resolve(DispatchPropertyDirectoryValuesToSyncAction::class)->execute();

    Queue::assertPushed(function (SyncPropertyDirectoryValuesJob $job) use ($values) {
        return arrayEquals($job->ids, $values->pluck('directory_value_id')->all());
    });

    assertDatabaseHas(SyncTimestamp::class, [
        'type' => SyncTimestampTypeEnum::CLOUD_PROPERTY_DIRECTORY_VALUES,
        'last_schedule' => now()->format((new SyncTimestamp())->getDateFormat()),
    ]);
});
