<?php

use App\Domain\Cloud\Jobs\SyncPropertyDirectoryValuesJob;
use App\Domain\Cloud\Jobs\Tests\TestCases\SyncPropertyDirectoryValuesJobTestCase;
use App\Domain\Offers\Models\PropertyDirectoryValue;
use Ensi\CloudApiSdk\Dto\Indexes\IndexesPropertyDirectoryValues\PropertyDirectoryValueAction\PropertyDirectoryValueActionEnum;
use Illuminate\Database\Eloquent\Collection;

use function Pest\Laravel\assertDatabaseHas;
use function PHPUnit\Framework\assertArrayNotHasKey;
use function PHPUnit\Framework\assertEquals;

uses(SyncPropertyDirectoryValuesJobTestCase::class);
uses()->group('component', 'SyncPropertyDirectoryValuesJob');

test("Action SyncPropertyDirectoryValuesJob integration disable", function () {
    /** @var SyncPropertyDirectoryValuesJobTestCase $this */
    $this->cloudIntegrationDisable();

    /** @var Collection<PropertyDirectoryValue> $propertyDirectoryValues */
    $propertyDirectoryValues = PropertyDirectoryValue::factory()->count(2)->create();

    $this->neverSendPropertyDirectoryValue();

    SyncPropertyDirectoryValuesJob::dispatchSync($propertyDirectoryValues->pluck('directory_value_id')->all());
});

test("Action SyncPropertyDirectoryValuesJob create|update success", function (bool $cloudCreated) {
    /** @var SyncPropertyDirectoryValuesJobTestCase $this */

    /** @var PropertyDirectoryValue $propertyDirectoryValue */
    $propertyDirectoryValue = PropertyDirectoryValue::factory()->create(['cloud_created' => $cloudCreated]);

    $action = &$this->shouldSendPropertyDirectoryValue();

    SyncPropertyDirectoryValuesJob::dispatchSync([$propertyDirectoryValue->directory_value_id]);

    assertEquals($cloudCreated ? PropertyDirectoryValueActionEnum::UPDATE : PropertyDirectoryValueActionEnum::CREATE, $action->action);

    assertDatabaseHas(PropertyDirectoryValue::class, [
        'id' => $propertyDirectoryValue->id,
        'cloud_created' => true,
    ]);
})->with([true, false]);

test("Action SyncPropertyDirectoryValuesJob base fields success", function () {
    /** @var SyncPropertyDirectoryValuesJobTestCase $this */

    /** @var PropertyDirectoryValue $propertyDirectoryValue */
    $propertyDirectoryValue = PropertyDirectoryValue::factory()->create();

    $action = &$this->shouldSendPropertyDirectoryValue();

    SyncPropertyDirectoryValuesJob::dispatchSync([$propertyDirectoryValue->directory_value_id]);

    expect($action->id)->toEqual($propertyDirectoryValue->directory_value_id)->toBeString();

    $body = $action->body->toArray();
    assertEquals($propertyDirectoryValue->name, $action->body->name);
    expect($action->body->property_id)->toEqual($propertyDirectoryValue->property_id)->toBeString();

    assertArrayNotHasKey('type', $body);
});

test("Action SyncPropertyDirectoryValuesJob delete success", function (bool $cloudCreated) {
    /** @var SyncPropertyDirectoryValuesJobTestCase $this */

    /** @var PropertyDirectoryValue $propertyDirectoryValue */
    $propertyDirectoryValue = PropertyDirectoryValue::factory()->trashed()->create([
        'cloud_created' => $cloudCreated,
    ]);

    if ($cloudCreated) {
        $action = &$this->shouldSendPropertyDirectoryValue();
    } else {
        $this->neverSendPropertyDirectoryValue();
    }

    SyncPropertyDirectoryValuesJob::dispatchSync([$propertyDirectoryValue->directory_value_id]);

    if ($cloudCreated) {
        assertEquals(PropertyDirectoryValueActionEnum::DELETE, $action->action);
        assertEquals($propertyDirectoryValue->directory_value_id, $action->id);
    }

    assertDatabaseHas(PropertyDirectoryValue::class, [
        'id' => $propertyDirectoryValue->id,
        'cloud_created' => false,
    ]);
})->with([true, false]);
