<?php

namespace App\Domain\Offers\Models;

use App\Domain\Offers\Models\Tests\Factories\PropertyFactory;
use Carbon\CarbonInterface;
use Illuminate\Database\Eloquent\Collection;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasMany;
use Illuminate\Database\Eloquent\SoftDeletes;

/**
 * @property int $id
 *
 * @property int $property_id ID from PIM
 *
 * @property string $name
 * @property string $code
 * @property string $type
 * @property bool $is_public
 * @property bool $is_active
 *
 * @property CarbonInterface|null $cloud_fields_updated_at
 * @property bool $cloud_created
 *
 * @property bool $is_migrated
 *
 * @property CarbonInterface|null $created_at
 * @property CarbonInterface|null $updated_at
 *
 * @property-read Collection<ActualCategoryProperty> $actualCategoryProperties
 */
class Property extends Model
{
    use SoftDeletes;

    protected $table = 'properties';

    protected $casts = [
        'property_id' => 'int',
        'is_public' => 'bool',
        'is_active' => 'bool',
        'is_migrated' => 'bool',
        'cloud_fields_updated_at' => 'datetime',
    ];

    public static function factory(): PropertyFactory
    {
        return PropertyFactory::new();
    }

    public function actualCategoryProperties(): HasMany
    {
        return $this->hasMany(ActualCategoryProperty::class, 'property_id', 'property_id');
    }
}
