<?php

namespace App\Domain\Offers\Observers;

use App\Domain\Offers\Actions\MarkProductAsUpdatedAction;
use App\Domain\Offers\Models\PropertyDirectoryValue;
use Illuminate\Database\Eloquent\Builder;

class PropertyDirectoryValueObserver
{
    public function __construct(
        protected MarkProductAsUpdatedAction $markAction
    ) {
    }

    public function creating(PropertyDirectoryValue $model): void
    {
        $model->cloud_fields_updated_at ??= now();
    }

    public function updating(PropertyDirectoryValue $model): void
    {
        if ($model->isDirty([
            'property_id',
            'name',
        ])) {
            $model->cloud_fields_updated_at = now();
        }
    }

    public function deleting(PropertyDirectoryValue $model): void
    {
        // It is not possible to save deleted_at during the update due to the implementation of soft-delete in the laravel kernel
        $model->cloud_fields_updated_at = now();
        $model->save();
    }

    public function saved(PropertyDirectoryValue $model): void
    {
        $this->markAction->execute(
            fn (Builder $query) => $query->whereHas('productPropertyValues', fn (Builder $queryValue) => $queryValue->where('directory_value_id', $model->directory_value_id)),
            $model->wasRecentlyCreated || $model->wasChanged(['value'])
        );
    }

    // Don't put a $markAction on deleted, because, ProductPropertyValue will be updated,
    // and through it there will be a mark for updating
}
