<?php

namespace App\Domain\Offers\Observers;

use App\Domain\Offers\Actions\MarkProductAsUpdatedAction;
use App\Domain\Offers\Models\Property;
use Illuminate\Database\Eloquent\Builder;

class PropertyObserver
{
    public function __construct(
        protected MarkProductAsUpdatedAction $markAction
    ) {
    }

    public function creating(Property $model): void
    {
        $model->cloud_fields_updated_at ??= now();
    }

    public function updating(Property $model): void
    {
        if ($model->isDirty([
            'name',
            'is_active',
        ])) {
            $model->cloud_fields_updated_at = now();
        }
    }

    public function deleting(Property $model): void
    {
        // It is not possible to save deleted_at during the update due to the implementation of soft-delete in the laravel kernel
        $model->cloud_fields_updated_at = now();
        $model->save();
    }

    public function saved(Property $model): void
    {
        $this->markAction->execute(
            fn (Builder $query) => $query->whereHas('productPropertyValues', fn (Builder $queryValue) => $queryValue->where('property_id', $model->property_id)),
            $model->wasRecentlyCreated || $model->wasChanged(['name', 'is_public', 'is_active'])
        );
    }

    // Don't put a $markAction on deleted, because, ProductPropertyValue will be updated,
    // and through it there will be a mark for updating
}
