<?php

use App\Domain\Offers\Models\ActualCategoryProperty;
use App\Http\ApiV1\OpenApiGenerated\Enums\PaginationTypeEnum;
use App\Http\ApiV1\Support\Tests\ApiV1ComponentTestCase;
use Illuminate\Support\Collection;

use function Pest\Laravel\postJson;

uses(ApiV1ComponentTestCase::class);
uses()->group('component');

test('POST /api/v1/categories/actual-category-properties:search success', function () {
    /** @var ApiV1ComponentTestCase $this */

    /** @var Collection<ActualCategoryProperty> $offers */
    $offers = ActualCategoryProperty::factory()
        ->forEachSequence(
            ['is_migrated' => false],
            ['is_migrated' => true],
        )
        ->create();

    postJson('/api/v1/categories/actual-category-properties:search', [
        "filter" => ["is_migrated" => true],
        "sort" => ["-id"],
    ])
        ->assertOk()
        ->assertJsonCount(1, 'data')
        ->assertJsonPath('data.0.id', $offers->last()->id)
        ->assertJsonPath('data.0.is_migrated', true);
});


test('POST /api/v1/categories/actual-category-properties:search filter success', function (
    string $fieldKey,
    mixed $value = null,
    ?string $filterKey = null,
    mixed $filterValue = null,
) {
    /** @var ApiV1ComponentTestCase $this */

    /** @var ActualCategoryProperty $offer */
    $offer = ActualCategoryProperty::factory()->create($value ? [$fieldKey => $value] : []);
    ActualCategoryProperty::factory()->create();

    postJson('/api/v1/categories/actual-category-properties:search', [
        "filter" => [($filterKey ?: $fieldKey) => ($filterValue ?: $offer->{$fieldKey}),],
        'sort' => ['id'],
        'pagination' => [
            'type' => PaginationTypeEnum::OFFSET,
            'limit' => 1,
        ],
    ])
        ->assertOk()
        ->assertJsonCount(1, 'data')
        ->assertJsonPath('data.0.id', $offer->id);
})->with([
    ['id'],
    ['actual_category_property_id'],
    ['property_id'],
    ['category_id'],
    ['is_migrated'],

    ['created_at', '2022-04-20', 'created_at_gte', '2022-04-19'],
    ['created_at', '2022-04-20', 'created_at_lte', '2022-04-21'],
    ['updated_at', '2022-04-20', 'updated_at_gte', '2022-04-19'],
    ['updated_at', '2022-04-20', 'updated_at_lte', '2022-04-21'],
]);

test("POST /api/v1/categories/actual-category-properties:search sort success", function (string $sort) {
    ActualCategoryProperty::factory()->create();
    postJson("/api/v1/categories/actual-category-properties:search", ["sort" => [$sort]])->assertOk();
})->with([
    'id',
    'is_migrated',
    'created_at',
    'updated_at',
]);
